#!/usr/bin/awk -f
#
# special_parser.awk - parsing script for awkiawki
#
# Copyright (c) 2002 Oliver Tonnhofer <olt@bogosoft.com>
# Copyright (c) 2003 Paul H. Liu <paul@thev.net>
#
# See the file `COPYING' for copyright notice.


BEGIN {
    index_anchor = 0
    index_level = 0
    scriptname = ENVIRON["SCRIPT_NAME"]
    wikiexp1 = "[A-Z]+[a-z]+[A-Z][A-Za-z]*|[A-Z][A-Z]+[a-z][A-Za-z]*"
    wikiexp2 = "_(\\w|[\\\\.:_-])+_"
    wikiexp = wikiexp1 "|" wikiexp2

    if (special_changes)
        print "<table>"
    if (special_history) {
        # use the power of awk to split the information of rlog
        RS = "----------------------------\n"
        FS = "\n"
        print "<table>\n<tr><th>version</th><th>date</th>" \
              "<th>ip</th><th>comment</th><th>view version</th>" \
              "<th>edit version</th></tr>" 
    }
} 

special_changes && ($9 ~ ("^" wikiexp "$") || 
$0 ~ ("^(" wikiexp ")(\\t([0-9a-fA-F]+))?$")) {
    if (match($0, /\t/) > 1) {
        page = substr($0, 1, RSTART - 1)
        rev = substr($0, RSTART + 1)
        t = strftime(date_fmt, rev)
    }
    else {
        page = $9
        rev = 1
        t = $7 " " $6 " " $8
    }
    filenr++
    print "<tr><td>" generate_link(page, rev) "</td><td>" t "</td></tr>"
    if (filenr == special_changes) 
      exit
}

special_index && ("^(" wikiexp ")(\\t([0-9a-fA-F]+))?$") {
    if (match($0, /\t/) > 1) {
        page = substr($0, 1, RSTART - 1)
        rev = substr($0, RSTART + 1)
    }
    else {
        page = $0
        rev = 1
    }
    print generate_link(page, rev) "<br>"
}

special_search && ("(" wikiexp ")$") {
    sub(/^.*[^\/]\//, "")
    print generate_link($0) "<br>"
}

special_rename && page && newpage && ("(" wikiexp ")$") {
    system("replace -s '" page "' '" newpage "' -- " $1)
#   print "replace '" page "' '" newpage "' -- " $1
    if (date && addr)
    {
        system("ci -q -t-" $1 " -l -m'" addr ";;" date \
               ";;affected by rename' " $1)
    }
}

special_history && NR > 1 {
    #pick revision number
    match($1, /[1-9]\.[0-9]+/)
    revision = substr($1, RSTART, RLENGTH)
    
    #split the double-semicolon seperated information line
    split($3, sp_array, ";;")
    ip_address = sp_array[1]
    date = sp_array[2]
    comment = sp_array[3]
    
    if (NR == 2) 
        revision = "current"
    
    print "<tr align=center><td>" revision "</td><td>" date \
          "</td><td>" ip_address "</td><td>" comment "</td>"
    print "<td><a href=\"" scriptname "/" special_history
    if (NR > 2)
        print "?revision=" revision
    print "\">view</a></td>"
    print "<td><a href=\"" scriptname "/?edit=true&amp;"
    if (NR >2)
        print "revision=" revision "&amp;"
    print "page=" special_history "\">edit</a></td></tr>"

}

special_uploadlist && !/No such file or directory/ {
    filesize = $5
    filename = $9
    for (i = 10; i <= NF; i++) { filename = filename " " $i; }

    if (length(filename) > 0 && !(filename ~ /~$/)) {
    print "<tr>"
    print "<td><a href=\"" datadir "UPLOAD/" page "/" filename \
          "\">" filename "</a></td>"
    print "<td>" filesize "</td>"
    print "<td align=center><a href=\"" scriptname \
          "?remove_upload=true&page=" page "&name=" \
          urlencode(filename) "\">yes</a></td>"
    print "</tr>"
    }
}

special_upload {
    getline
    if ($0 ~ /filename=/) {
        match($0, /filename=/)
        uploadfilename = substr($0, RSTART + RLENGTH + 1, \
                                length($0) - RSTART - RLENGTH - 2)
    }
    if (uploadfilename == "") { print "\t0"; exit 0; }
    
    getline
    getline
    BINMODE = "rw"
    gsub(/\+/, "\\+", boundary)
    RS = "\r\n--" boundary "--"
    getline
    ORS = ""
    OFS = ""
    print $0 > tempfile
    print uploadfilename "\t" length($0)
    exit 0
}

special_summary && /^-[^-]/ {
    close_ul(1)
    open_ul(1)
    print "<li><a href=\"#anchor_" index_anchor "\">" substr($0, 2) \
          "</a></li>"
    index_anchor = index_anchor + 1
    next;
}

special_summary && /^--[^-]/ {
    close_ul(2)
    open_ul(2)
    print "<li><a href=\"#anchor_" index_anchor "\">" substr($0, 3) \
          "</a></li>"
    index_anchor = index_anchor + 1
    next;
}

special_summary && /^---[^-]/ {
    close_ul(3)
    open_ul(3)
    print "<li><a href=\"anchor_" index_anchor "\">" substr($0, 4) \
          "</a></li>"
    index_anchor = index_anchor + 1
    next; 
}

END {
    if (special_changes || special_history)
        print "</table>"
    if (special_summary)
        close_ul(0)
}

function generate_link(string, rev) {
    if (string ~ ("^" wikiexp1 "$"))
    {
        sub("^" wikiexp1 "$", \
            "<a href=\""scriptname"/&?stamp=" rev "\">&</a>", string)
    }
    else if (string ~ (wikiexp2)) {
        match(string, wikiexp2)
        visname = substr(string, RSTART + 1, RLENGTH - 2)
        gsub(/_/, " ", visname)
        sub(wikiexp2, 
            "<a href=\"" scriptname "/&?stamp=" rev "\">" visname "</a>", 
            string)
    }
    return(string)
}

function urlencode(string) {
# We assume an awk implementation that is just plain dumb.
# We will convert an character to its ASCII value with the
# table ord[], and produce two-digit hexadecimal output
# without the printf("%02X") feature.

    EOL = "%0A"                # "end of line" string (encoded)
    split ("1 2 3 4 5 6 7 8 9 A B C D E F", hextab, " ")
    hextab [0] = 0
    for (i = 1; i <= 255; ++i ) ord [ sprintf ("%c", i) "" ] = i + 0

    encoded = ""
    for (i = 1; i <= length (string); ++i) {
        c = substr (string, i, 1)
        if ( c ~ /[a-zA-Z0-9.-]/ ) {
            encoded = encoded c# safe character
        } else if ( c == " " ) {
            encoded = encoded "+"# special handling
        } else {
            # unsafe character, encode it as a two-digit hex-number
            lo = ord [c] % 16
            hi = int (ord [c] / 16);
            encoded = encoded "%" hextab [hi] hextab [lo]
        }
    }
    return encoded
}

function close_ul(l)
{
    while (index_level > l) { 
    print("</ul>"); index_level = index_level - 1;
    }
}

function open_ul(l)
{
    while (index_level < l) { 
    print("<ul>"); index_level = index_level + 1;
    }
}
